@@ -26,7 +26,7 @@ sub import {
 	# Hook to change to do pinging since it's called late.
 	# This ensures each page only pings once and prevents slow
 	# pings interrupting page builds.
-	hook(type => "change", id => "inline", call => \&IkiWiki::pingurl);
+	hook(type => "rendered", id => "inline", call => \&IkiWiki::pingurl);
 }
 
 sub getopt () {
@@ -119,7 +119,7 @@ sub sessioncgi ($$) {
 	my $session=shift;
 
 	if ($q->param('do') eq 'blog') {
-		my $page=titlepage(decode_utf8($q->param('title')));
+		my $page=titlepage(decode_utf8(scalar $q->param('title')));
 		$page=~s/(\/)/"__".ord($1)."__"/eg; # don't create subdirs
 		# if the page already exists, munge it to be unique
 		my $from=$q->param('from');
@@ -160,16 +160,17 @@ sub preprocess_inline (@) {
 		# Running in scan mode: only do the essentials
 
 		if (yesno($params{trail}) && IkiWiki::Plugin::trail->can("preprocess_trailitems")) {
-			# default to sorting age, the same as inline itself,
-			# but let the params override that
-			IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age', %params);
+			# default to sorting by age with fallback to title,
+			# the same as inline itself, but let the params
+			# override that
+			IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age title', %params);
 		}
 
 		return;
 	}
 
 	if (yesno($params{trail}) && IkiWiki::Plugin::trail->can("preprocess_trailitems")) {
-		scalar IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age', %params);
+		scalar IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age title', %params);
 	}
 
 	my $raw=yesno($params{raw});
@@ -186,6 +187,13 @@ sub preprocess_inline (@) {
 	if (! exists $params{feedshow} && exists $params{show}) {
 		$params{feedshow}=$params{show};
 	}
+	my $title;
+	if (exists $params{title}) {
+		$title = $params{title};
+	}
+	else {
+		$title = $params{page} ne "index" ? pagetitle($params{page}) : $config{wikiname};
+	}
 	my $desc;
 	if (exists $params{description}) {
 		$desc = $params{description} 
@@ -211,8 +219,7 @@ sub preprocess_inline (@) {
 			}
 		}
 
-		@list = map { bestlink($params{page}, $_) }
-		        split ' ', $params{pagenames};
+		@list = split ' ', $params{pagenames};
 
 		if (yesno($params{reverse})) {
 			@list=reverse(@list);
@@ -221,6 +228,8 @@ sub preprocess_inline (@) {
 		foreach my $p (@list) {
 			add_depends($params{page}, $p, deptype($quick ? "presence" : "content"));
 		}
+
+		@list = grep { exists $pagesources{$_} } @list;
 	}
 	else {
 		my $num=0;
@@ -237,7 +246,7 @@ sub preprocess_inline (@) {
 		@list = pagespec_match_list($params{page}, $params{pages},
 			deptype => deptype($quick ? "presence" : "content"),
 			filter => sub { $_[0] eq $params{page} },
-			sort => exists $params{sort} ? $params{sort} : "age",
+			sort => exists $params{sort} ? $params{sort} : "age title",
 			reverse => yesno($params{reverse}),
 			($num ? (num => $num) : ()),
 		);
@@ -321,8 +330,12 @@ sub preprocess_inline (@) {
 
 	my $ret="";
 
-	if (length $config{cgiurl} && ! $params{preview} && (exists $params{rootpage} ||
-	    (exists $params{postform} && yesno($params{postform}))) &&
+	my $postform = (exists $params{rootpage});
+	if (exists $params{postform}) {
+		$postform = yesno($params{postform});
+	}
+
+	if (length $config{cgiurl} && ! $params{preview} && $postform &&
 	    IkiWiki->can("cgi_editpage")) {
 		# Add a blog post form, with feed buttons.
 		my $formtemplate=template_depends("blogpost.tmpl", $params{page}, blind_cache => 1);
@@ -383,7 +396,9 @@ sub preprocess_inline (@) {
 					blind_cache => 1);
 			};
 			if ($@) {
-				error sprintf(gettext("failed to process template %s"), $params{template}.".tmpl").": $@";
+				# gettext can clobber $@
+				my $error = $@;
+				error sprintf(gettext("failed to process template %s"), $params{template}.".tmpl").": $error";
 			}
 		}
 		my $needcontent=$raw || (!($archive && $quick) && $template->query(name => 'content'));
@@ -464,7 +479,7 @@ sub preprocess_inline (@) {
 			if (! $params{preview}) {
 				writefile($rssp, $config{destdir},
 					genfeed("rss",
-						$config{url}."/".$rssp, $desc, $params{guid}, $params{page}, @feedlist));
+						$config{url}."/".$rssp, $title, $desc, $params{guid}, $params{page}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
 				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/rss+xml" title="$rssdesc" href="$rssurl" />};
 			}
@@ -474,7 +489,7 @@ sub preprocess_inline (@) {
 			will_render($params{destpage}, $atomp);
 			if (! $params{preview}) {
 				writefile($atomp, $config{destdir},
-					genfeed("atom", $config{url}."/".$atomp, $desc, $params{guid}, $params{page}, @feedlist));
+					genfeed("atom", $config{url}."/".$atomp, $title, $desc, $params{guid}, $params{page}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
 				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/atom+xml" title="$atomdesc" href="$atomurl" />};
 			}
@@ -610,9 +625,30 @@ sub absolute_urls ($$) {
 	return $ret;
 }
 
+sub genenclosure {
+	my $itemtemplate=shift;
+	my $url=shift;
+	my $file=shift;
+
+	return unless $itemtemplate->query(name => "enclosure");
+
+	my $size=(srcfile_stat($file))[8];
+	my $mime="unknown";
+	eval q{use File::MimeInfo};
+	if (! $@) {
+		$mime = mimetype($file);
+	}
+	$itemtemplate->param(
+		enclosure => $url,
+		type => $mime,
+		length => $size,
+	);
+}
+
 sub genfeed ($$$$$@) {
 	my $feedtype=shift;
 	my $feedurl=shift;
+        my $feedtitle=shift;
 	my $feeddesc=shift;
 	my $guid=shift;
 	my $page=shift;
@@ -626,6 +662,7 @@ sub genfeed ($$$$$@) {
 	foreach my $p (@pages) {
 		my $u=URI->new(encode_utf8(urlto($p, "", 1)));
 		my $pcontent = absolute_urls(get_inline_content($p, $page), $url);
+		my $fancy_enclosure_seen = 0;
 
 		$itemtemplate->param(
 			title => pagetitle(basename($p)),
@@ -647,32 +684,23 @@ sub genfeed ($$$$$@) {
 				$itemtemplate->param(mdate_822 => date_822($pagestate{$p}{meta}{updated}));
 				$itemtemplate->param(mdate_3339 => date_3339($pagestate{$p}{meta}{updated}));
 			}
+
+			if (exists $pagestate{$p}{meta}{enclosure}) {
+				my $absurl = $pagestate{$p}{meta}{enclosure};
+				my $file = $pagestate{$p}{meta}{enclosurefile};
+				genenclosure($itemtemplate, $absurl, $file);
+				$fancy_enclosure_seen = 1;
+			}
 		}
 
-		if ($itemtemplate->query(name => "enclosure")) {
-			my $file=$pagesources{$p};
-			my $type=pagetype($file);
-			if (defined $type) {
-				$itemtemplate->param(content => $pcontent);
-			}
-			else {
-				my $size=(srcfile_stat($file))[8];
-				my $mime="unknown";
-				eval q{use File::MimeInfo};
-				if (! $@) {
-					$mime = mimetype($file);
-				}
-				$itemtemplate->param(
-					enclosure => $u,
-					type => $mime,
-					length => $size,
-				);
-			}
-		}
-		else {
-			$itemtemplate->param(content => $pcontent);
+		my $file=$pagesources{$p};
+		unless ($fancy_enclosure_seen || defined(pagetype($file))) {
+			genenclosure($itemtemplate, $u, $file);
+			$itemtemplate->param(simplepodcast => 1);
 		}
 
+		$itemtemplate->param(content => $pcontent);
+
 		run_hooks(pagetemplate => sub {
 			shift->(page => $p, destpage => $page,
 				template => $itemtemplate);
@@ -686,13 +714,14 @@ sub genfeed ($$$$$@) {
 
 	my $template=template_depends($feedtype."page.tmpl", $page, blind_cache => 1);
 	$template->param(
-		title => $page ne "index" ? pagetitle($page) : $config{wikiname},
+		title => $feedtitle,
 		wikiname => $config{wikiname},
 		pageurl => $url,
 		content => $content,
 		feeddesc => $feeddesc,
 		guid => $guid,
 		feeddate => date_3339($lasttime),
+		feeddate_822 => date_822($lasttime),
 		feedurl => $feedurl,
 	);
 	run_hooks(pagetemplate => sub {
